#ifndef RTOS_H
#define RTOS_H

#include "buildconf.h"

#define BYPASS_DC           1

/* SPM - below SPM_LIMIT */
#define SPM_LIMIT           0x00010000


/* Code and user data area: SPM_LIMIT to 0x80000000 */
#define SYSTEM_STACK        0x9ffffff0
#define SW_LOG_BASE         0x9f000000
#define MAIN_MEMORY_START   0x90000000
#define MAIN_MEMORY_END     0xa0000000
#define DMA_SPM_START       0x10000000
#define CALL_STACK          0x9e000000


/* Devices area */
#define TIMER_REGISTER      0x80010000
#define BASE_REGISTER_AREA  0x80020000
#define UART_BASE           0x84000000 /* UartLite */

#define RUN_FOR             0xf0000000U

#define CLOCK_PERIOD        10 /* ns */

/* CPU flags */
#ifndef MSR_IE
#define MSR_IE              0x002
#endif
#ifndef MSR_BE
#define MSR_BE              0x100
#endif
#define MSR_MASK            (MSR_IE | MSR_BE)
#define MODE_USER           (MSR_IE | MSR_BE)
#define MODE_SYSCALL        (0)
#define MODE_SYSTEM         (MSR_BE)

#define REGION_CHANGE       0x30

#define BLOCK_SIZE_SHIFT    4
#define NUM_BLOCKS_SHIFT    7
#define MAX_THREADS         16



#define NUM_BLOCKS          (1 << NUM_BLOCKS_SHIFT)
#define BLOCK_SIZE          (1 << BLOCK_SIZE_SHIFT)

#define EXT_MEM_DMA_TIME    32
#define EXT_IC_MISS         29

#define EXT_MEM_LATENCY     15
#define EXT_MEM_BUS_WIDTH   4
#define EXT_MEM_TRANS_BYTES BLOCK_SIZE

#define ICACHE_STORE_SIZE   (BLOCK_SIZE * NUM_BLOCKS)
#define DCACHE_STORE_SIZE   8192 // (BLOCK_SIZE * NUM_BLOCKS)
#define ICACHE_LINE_SIZE    BLOCK_SIZE
#define DCACHE_LINE_SIZE    16
/*#define UNLIMITED_TRANSFERS*/

#define OPEN_DISCARD_FLAG   0x4000
#define OPEN_SCRATCH_FLAG   0x2000
#define OPEN_DEFER_FLAG     0x1000
#define OPEN_SIZE_BITS      0x00ff

#define AKD __attribute__ ((section (".kernelstuff.d")))
#define AKT __attribute__ ((section (".kernelstuff.t"))) 
#define ALIGN __attribute__ ((aligned(BLOCK_SIZE)))


#define EXIT_CRASH                  10
#define EXIT_NO_TASK                20
#define EXIT_RUN_FOR                60
#define EXIT_DEADLINE_OVERRUN       70

#define CMD_EXIT                    0x80000001
#define CMD_HALT                    0x80000002
#define CMD_PRNT                    0x80000003
#define CMD_NTFB                    0x80000004
#define CMD_TFIN                    0x80000005
#define CMD_RTFI                    0x80000006
#define CMD_IDLE                    0x80000007
#define CMD_NAME                    0x80000008
#define CMD_CALL                    0x80000009
#define CMD_RTFF                    0x8000000a


#define BIW(sspm, sr0)              ((((sspm)-1)*BLOCK_SIZE) | \
                                        ((((sr0)-1)*BLOCK_SIZE) << 16))
#define DEFAULT_CALL_BIW            BIW(NUM_BLOCKS, NUM_BLOCKS)
#define DEFAULT_LOAD_BIW            DEFAULT_CALL_BIW

#ifndef FROM_ASM

/* defined in asm */
extern unsigned call_function(void * fn, unsigned block_info_word, ...);
extern void * memcpy(void * dest, const void * src, unsigned size);
extern void * memmove(void * dest, const void * src, unsigned size);
extern void _open(void * base, unsigned block_number, unsigned num_blocks);
extern void print(const char * text);
extern void * scheduler(void);
extern void exit(int rc);
extern void _exit(int rc);
extern void notify_task_name(const char * name, void * function);
extern void set_trigger(unsigned);
extern unsigned get_time(void);
extern volatile unsigned char __end_spm;
extern unsigned get_hires_timer(void);

typedef void (* FN) (void);
extern unsigned spm_loader(unsigned spm_block, FN ext_mem_src, unsigned size);

/* defined here */
#define _close(block_number, num_blocks) \
    _open((void *) 0xffffffff, (block_number), (num_blocks))
#define open(handle, base, size) \
    do { const unsigned num_blocks = ((size) + BLOCK_SIZE - 1) / BLOCK_SIZE; \
    _open((void *) (base), handle, (num_blocks)); \
    handle += num_blocks; } while(0)
#define close(handle, base, size) \
    do { const unsigned num_blocks = ((size) + BLOCK_SIZE - 1) / BLOCK_SIZE; \
    handle -= num_blocks; _close(handle, (num_blocks)); } while(0)

/* defined in C */
void * scheduler(void) AKT;
void register_task(const char * name, void * function, 
                unsigned block_info_word,
                unsigned offset, unsigned period) AKT;
void kernel_start(void) AKT;
void crash(void) AKT;
void printhex(unsigned x) AKT;
const char * debug_find_task(void * x);
void task_func_is_zero(void);
void printdec(unsigned x);
void cs_measure(unsigned n, unsigned search_size,
            unsigned call_overhead, unsigned return_overhead);

#endif
#endif

